/*
    SPDX-FileCopyrightText: 2005-2006 Olivier Goffart <ogoffart at kde.org>

    SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
*/

#include "knotificationplugin.h"

KNotificationPlugin::KNotificationPlugin(QObject *parent, const QVariantList &args)
    : QObject(parent),
      d(nullptr)
{
    Q_UNUSED(args);
}


KNotificationPlugin::~KNotificationPlugin()
{
}


void KNotificationPlugin::update(KNotification *notification, KNotifyConfig *config)
{
    Q_UNUSED(notification);
    Q_UNUSED(config);
}

void KNotificationPlugin::close(KNotification *notification)
{
    emit finished(notification);
}

void KNotificationPlugin::finish(KNotification *notification)
{
    emit finished(notification);
}

